@@ -166,7 +166,7 @@ fallback_shape (PangoEngineShape *engine,
 
       if (pango_is_zero_width (wc))
 	{
-	  set_glyph (font, glyphs, i, p - text, 0);
+	  set_glyph (font, glyphs, i, p - text, PANGO_GLYPH_EMPTY);
 	}
       else
 	{
@@ -174,7 +174,7 @@ fallback_shape (PangoEngineShape *engine,
 
 	  if (!index)
             {
-	      index = pango_fc_font_get_unknown_glyph (fc_font, wc);
+	      index = PANGO_GET_UNKNOWN_GLYPH ( wc);
               set_glyph (font, glyphs, i, p - text, index);
 	    }
 	  else
@@ -382,7 +382,7 @@ basic_engine_shape (PangoEngineShape *engine,
 
       if (pango_is_zero_width (wc))	/* Zero-width characters */
 	{
-	  pango_ot_buffer_add_glyph (buffer, 0, unknown_property, p - text);
+	  pango_ot_buffer_add_glyph (buffer, PANGO_GLYPH_EMPTY, unknown_property, p - text);
 	}
       else
         {
@@ -390,7 +390,7 @@ basic_engine_shape (PangoEngineShape *engine,
 	  
 	  if (!index)
 	    {
-	      pango_ot_buffer_add_glyph (buffer, pango_fc_font_get_unknown_glyph (fc_font, wc),
+	      pango_ot_buffer_add_glyph (buffer, PANGO_GET_UNKNOWN_GLYPH ( wc),
 					 unknown_property, p - text);
 	    }
 	  else
