@@ -128,7 +128,7 @@ set_glyph_tone (PangoFont *font, PangoGlyphString *glyphs, int i,
 #define find_char(font,wc) \
     pango_fc_font_get_glyph((PangoFcFont *)font, wc)
 #define get_unknown_glyph(font,wc) \
-    pango_fc_font_get_unknown_glyph((PangoFcFont *)font, wc)
+    PANGO_GET_UNKNOWN_GLYPH ( wc)
 
 static void
 render_tone (PangoFont *font, gunichar tone, PangoGlyphString *glyphs,
@@ -171,7 +171,7 @@ render_isolated_tone (PangoFont *font, gunichar tone, PangoGlyphString *glyphs,
     index = find_char (font, 0x25cb);   /* WHITE CIRCLE, in KSC-5601 */
   if (!index)
     index = find_char (font, ' ');      /* Space */
-  if (!index)			        /* Unknown glyph box with 0000 in it */
+  if (!index)				/* Unknown glyph box with 0000 in it */
     index = find_char (font, get_unknown_glyph (font, 0));
 
   /* Add the base character
@@ -285,17 +285,17 @@ render_basic (PangoFont *font, gunichar wc,
   if (wc == 0xa0)	/* non-break-space */
     wc = 0x20;
 
-  index = find_char (font, wc);
   pango_glyph_string_set_size (glyphs, *n_glyphs + 1);
-  if (index)
-    set_glyph (font, glyphs, *n_glyphs, cluster_offset, index);
+
+  if (pango_is_zero_width (wc))
+    set_glyph (font, glyphs, *n_glyphs, cluster_offset, PANGO_GLYPH_EMPTY);
   else
     {
-      if (pango_is_zero_width (wc))
-	set_glyph (font, glyphs, *n_glyphs, cluster_offset, 0);
+      index = find_char (font, wc);
+      if (index)
+	set_glyph (font, glyphs, *n_glyphs, cluster_offset, index);
       else
-	set_glyph (font, glyphs, *n_glyphs, cluster_offset,
-		   get_unknown_glyph (font, wc));
+	set_glyph (font, glyphs, *n_glyphs, cluster_offset, get_unknown_glyph (font, wc));
     }
   (*n_glyphs)++;
 }
