@@ -474,7 +474,7 @@ get_index (PangoFcFont *fc_font, gunichar wc)
 {
   PangoGlyph index = pango_fc_font_get_glyph (fc_font, wc);
   if (!index)
-    index = pango_fc_font_get_unknown_glyph (fc_font, wc);
+    index = PANGO_GET_UNKNOWN_GLYPH ( wc);
   return index;
 }
 
