@@ -307,7 +307,7 @@ shape_ellipsis (EllipsizeState *state)
    */
   if (!state->ellipsis_run)
     {
-      state->ellipsis_run = g_new (PangoGlyphItem, 1);
+      state->ellipsis_run = g_slice_new (PangoGlyphItem);
       state->ellipsis_run->glyphs = pango_glyph_string_new ();
       state->ellipsis_run->item = NULL;
     }
