@@ -33,7 +33,7 @@ pango_attr_type_get_type (void)
       { PANGO_ATTR_ABSOLUTE_SIZE, "PANGO_ATTR_ABSOLUTE_SIZE", "absolute-size" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoAttrType", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoAttrType"), values);
   }
   return etype;
 }
@@ -50,7 +50,7 @@ pango_underline_get_type (void)
       { PANGO_UNDERLINE_ERROR, "PANGO_UNDERLINE_ERROR", "error" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoUnderline", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoUnderline"), values);
   }
   return etype;
 }
@@ -68,7 +68,7 @@ pango_coverage_level_get_type (void)
       { PANGO_COVERAGE_EXACT, "PANGO_COVERAGE_EXACT", "exact" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoCoverageLevel", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoCoverageLevel"), values);
   }
   return etype;
 }
@@ -85,7 +85,7 @@ pango_style_get_type (void)
       { PANGO_STYLE_ITALIC, "PANGO_STYLE_ITALIC", "italic" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoStyle", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoStyle"), values);
   }
   return etype;
 }
@@ -99,7 +99,7 @@ pango_variant_get_type (void)
       { PANGO_VARIANT_SMALL_CAPS, "PANGO_VARIANT_SMALL_CAPS", "small-caps" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoVariant", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoVariant"), values);
   }
   return etype;
 }
@@ -118,7 +118,7 @@ pango_weight_get_type (void)
       { PANGO_WEIGHT_HEAVY, "PANGO_WEIGHT_HEAVY", "heavy" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoWeight", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoWeight"), values);
   }
   return etype;
 }
@@ -139,7 +139,7 @@ pango_stretch_get_type (void)
       { PANGO_STRETCH_ULTRA_EXPANDED, "PANGO_STRETCH_ULTRA_EXPANDED", "ultra-expanded" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoStretch", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoStretch"), values);
   }
   return etype;
 }
@@ -157,7 +157,7 @@ pango_font_mask_get_type (void)
       { PANGO_FONT_MASK_SIZE, "PANGO_FONT_MASK_SIZE", "size" },
       { 0, NULL, NULL }
     };
-    etype = g_flags_register_static ("PangoFontMask", values);
+    etype = g_flags_register_static (g_intern_static_string ("PangoFontMask"), values);
   }
   return etype;
 }
@@ -174,7 +174,7 @@ pango_alignment_get_type (void)
       { PANGO_ALIGN_RIGHT, "PANGO_ALIGN_RIGHT", "right" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoAlignment", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoAlignment"), values);
   }
   return etype;
 }
@@ -189,7 +189,7 @@ pango_wrap_mode_get_type (void)
       { PANGO_WRAP_WORD_CHAR, "PANGO_WRAP_WORD_CHAR", "word-char" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoWrapMode", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoWrapMode"), values);
   }
   return etype;
 }
@@ -205,7 +205,7 @@ pango_ellipsize_mode_get_type (void)
       { PANGO_ELLIPSIZE_END, "PANGO_ELLIPSIZE_END", "end" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoEllipsizeMode", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoEllipsizeMode"), values);
   }
   return etype;
 }
@@ -223,7 +223,7 @@ pango_render_part_get_type (void)
       { PANGO_RENDER_PART_STRIKETHROUGH, "PANGO_RENDER_PART_STRIKETHROUGH", "strikethrough" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoRenderPart", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoRenderPart"), values);
   }
   return etype;
 }
@@ -299,7 +299,7 @@ pango_script_get_type (void)
       { PANGO_SCRIPT_KHAROSHTHI, "PANGO_SCRIPT_KHAROSHTHI", "kharoshthi" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoScript", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoScript"), values);
   }
   return etype;
 }
@@ -314,7 +314,7 @@ pango_tab_align_get_type (void)
       { PANGO_TAB_LEFT, "PANGO_TAB_LEFT", "left" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoTabAlign", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoTabAlign"), values);
   }
   return etype;
 }
@@ -335,7 +335,7 @@ pango_direction_get_type (void)
       { PANGO_DIRECTION_NEUTRAL, "PANGO_DIRECTION_NEUTRAL", "neutral" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static ("PangoDirection", values);
+    etype = g_enum_register_static (g_intern_static_string ("PangoDirection"), values);
   }
   return etype;
 }
