@@ -64,7 +64,8 @@ init_tabs (PangoTabArray *array, gint start, gint end)
  * pixel units if @positions_in_pixels is %TRUE, otherwise in Pango
  * units. All stops are initially at position 0.
  * 
- * Return value: the new #PangoTabArray
+ * Return value: the newly allocated #PangoTabArray, which should
+ *               be freed with pango_tab_array_free().
  **/
 PangoTabArray*
 pango_tab_array_new (gint initial_size,
@@ -79,7 +80,7 @@ pango_tab_array_new (gint initial_size,
    * If we allowed tab array resizing we'd need to drop this
    * optimization.
    */
-  array = g_new (PangoTabArray, 1);
+  array = g_slice_new (PangoTabArray);
   array->size = initial_size;
   array->allocated = initial_size;
 
@@ -109,7 +110,8 @@ pango_tab_array_new (gint initial_size,
  * tab stop. You <emphasis>must</emphasis> provide an alignment
  * and position for @size tab stops.
  * 
- * Return value: the new #PangoTabArray
+ * Return value: the newly allocated #PangoTabArray, which should
+ *               be freed with pango_tab_array_free().
  **/
 PangoTabArray  *
 pango_tab_array_new_with_positions (gint           size,
@@ -172,7 +174,8 @@ pango_tab_array_get_type (void)
  * 
  * Copies a #PangoTabArray
  * 
- * Return value: the new #PangoTabArray.
+ * Return value: the newly allocated #PangoTabArray, which should
+ *               be freed with pango_tab_array_free().
  **/
 PangoTabArray*
 pango_tab_array_copy (PangoTabArray *src)
@@ -202,7 +205,7 @@ pango_tab_array_free   (PangoTabArray *tab_array)
 
   g_free (tab_array->tabs);
   
-  g_free (tab_array);
+  g_slice_free (PangoTabArray, tab_array);
 }
 
 /**
