@@ -31,8 +31,6 @@
 #include "pangowin32.h"
 #include "pangowin32-private.h"
 
-#define PANGO_WIN32_UNKNOWN_FLAG 0x10000000
-
 #define CH_IS_UNIHAN_BMP(ch) ((ch) >= 0x3400 && (ch) <= 0x9FFF)
 #define CH_IS_UNIHAN(ch) (CH_IS_UNIHAN_BMP (ch) || \
 			  ((ch) >= 0x20000 && (ch) <= 0x2A6DF) ||	\
@@ -313,11 +311,12 @@ pango_win32_render (HDC               hdc,
        * point zero (just spacing).
        */
       while (i < glyphs->num_glyphs &&
-	     (glyphs->glyphs[i].glyph == 0 || cur_y_offset == glyphs->glyphs[i].geometry.y_offset))
+	     (glyphs->glyphs[i].glyph == PANGO_GLYPH_EMPTY ||
+	      cur_y_offset == glyphs->glyphs[i].geometry.y_offset))
 	{
-	  if (glyphs->glyphs[i].glyph == 0)
+	  if (glyphs->glyphs[i].glyph == PANGO_GLYPH_EMPTY)
 	    {
-	      /* Code point 0 glyphs should not be rendered, but their
+	      /* PANGO_GLYPH_EMPTY glyphs should not be rendered, but their
 	       * indicated width (set up by PangoLayout) should be taken
 	       * into account.
 	       */
@@ -337,7 +336,7 @@ pango_win32_render (HDC               hdc,
 	    }
 	  else
 	    {
-	      if (glyphs->glyphs[i].glyph & PANGO_WIN32_UNKNOWN_FLAG)
+	      if (glyphs->glyphs[i].glyph & PANGO_GLYPH_UNKNOWN_FLAG)
 		{
 		  /* Glyph index is actually the char value that doesn't
 		   * have any glyph (ORed with the flag). We should really
@@ -462,24 +461,23 @@ pango_win32_font_get_glyph_extents (PangoFont      *font,
   MAT2 m = {{0,1}, {0,0}, {0,0}, {0,1}};
   PangoWin32GlyphInfo *info;
 
-  if (glyph & PANGO_WIN32_UNKNOWN_FLAG)
+  if (glyph == PANGO_GLYPH_EMPTY)
+    {
+      if (ink_rect)
+	ink_rect->x = ink_rect->width = ink_rect->y = ink_rect->height = 0;
+      if (logical_rect)
+	logical_rect->x = logical_rect->width = logical_rect->y = logical_rect->height = 0;
+      return;
+    }
+
+  if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
     glyph_index = glyph = 0;
 
   info = g_hash_table_lookup (win32font->glyph_info, GUINT_TO_POINTER (glyph));
   
   if (!info)
     {
-      info = g_new (PangoWin32GlyphInfo, 1);
-
-      info->ink_rect.x = 0;
-      info->ink_rect.width = 0;
-      info->ink_rect.y = 0;
-      info->ink_rect.height = 0;
-
-      info->logical_rect.x = 0;
-      info->logical_rect.width = 0;
-      info->logical_rect.y = 0;
-      info->logical_rect.height = 0;
+      info = g_new0 (PangoWin32GlyphInfo, 1);
 
       memset (&gm, 0, sizeof (gm));
 
@@ -862,6 +860,8 @@ pango_win32_font_find_shaper (PangoFont     *font,
  * 
  * Returns the index of a glyph suitable for drawing @wc as an
  * unknown character.
+ *
+ * Use PANGO_GET_UNKNOWN_GLYPH() instead.
  * 
  * Return value: a glyph index into @font
  **/
@@ -869,7 +869,7 @@ PangoGlyph
 pango_win32_get_unknown_glyph (PangoFont *font,
 			       gunichar   wc)
 {
-  return wc | PANGO_WIN32_UNKNOWN_FLAG;
+  return PANGO_GET_UNKNOWN_GLYPH (wc);
 }
 
 /**
@@ -1367,7 +1367,8 @@ font_get_cmap (PangoFont *font)
  * @font: a #PangoFont.
  * @wc: a Unicode character.
  *
- * Obtains the index of the glyph for @wc in @font.
+ * Obtains the index of the glyph for @wc in @font, or 0, if not
+ * covered.
  *
  * Return value: the glyph index for @wc.
  **/
