@@ -33,8 +33,6 @@
 #include "pangox.h"
 #include "pangox-private.h"
 
-#define PANGO_X_UNKNOWN_FLAG 0x10000000
-
 #define PANGO_TYPE_X_FONT              (pango_x_font_get_type ())
 #define PANGO_X_FONT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_X_FONT, PangoXFont))
 #define PANGO_X_FONT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_X_FONT, PangoXFontClass))
@@ -401,31 +399,85 @@ pango_x_render  (Display           *display,
        * the ink rect here would be a noticeable speed hit.
        * This is close enough.
        */
-      if (!(glyph &&
+      if (!(glyph != PANGO_GLYPH_EMPTY &&
 	    glyph_x >= -16384 && glyph_x <= 32767 &&
 	    glyph_y >= -16384 && glyph_y <= 32767))
 	goto next_glyph;
 	      
-      if (glyph & PANGO_X_UNKNOWN_FLAG)
-        {
-	  PangoFontMetrics *metrics = pango_font_get_metrics (font,
-							      pango_language_from_string ("en"));
+      if ((glyph & PANGO_GLYPH_UNKNOWN_FLAG) == 0)
+	{
+	  guint16 index = PANGO_X_GLYPH_INDEX (glyph);
+	  guint16 subfont_index = PANGO_X_GLYPH_SUBFONT (glyph);
+	  PangoXSubfontInfo *subfont;
+      
+	  subfont = pango_x_find_subfont (font, subfont_index);
+	  if (subfont)
+	    {
+	      fs = pango_x_get_font_struct (font, subfont);
+	      if (!fs)
+		continue;
+	      
+	      if (fs->fid != old_fid)
+		{
+		  FLUSH;
+		  XSetFont (display, gc, fs->fid);
+		  old_fid = fs->fid;
+		}
+
+	      if (charcount == G_N_ELEMENTS (xcharbuffer) ||
+		  (charcount > 0 && (glyph_y != glyph_y0 ||
+				     glyph_x != expected_x)))
+		FLUSH;
+
+	      if (charcount == 0)
+		{
+		  glyph_x0 = glyph_x;
+		  glyph_y0 = glyph_y;
+		}
+	      xcharbuffer[charcount].byte1 = index / 256;
+	      xcharbuffer[charcount].byte2 = index % 256;
+
+	      expected_x = glyph_x + XTextWidth16 (fs, &xcharbuffer[charcount], 1);
+
+	      charcount++;
+	    } else
+	      goto unknown_glyph;
+	} else {
+	  PangoFontMetrics *metrics;
           int x1, y1, x2, y2; /* rectangle the character should go inside. */
           int baseline;
 	  int stroke_thick;
-
           gunichar wc;
-
+	  
+	unknown_glyph:
 	  FLUSH;
 
-          x1 = glyph_x;
-          y1 = glyph_y - PANGO_PIXELS (metrics->ascent);
-          x2 = x1 + PANGO_PIXELS (glyphs->glyphs[i].geometry.width);
-          y2 = y1 + PANGO_PIXELS (metrics->ascent + metrics->descent);
+	  if (font)
+	    metrics = pango_font_get_metrics (font, NULL);
+	  else
+	    metrics = NULL;
+
+	  if (metrics)
+	    {
+	      y1 = glyph_y - PANGO_PIXELS (metrics->ascent);
+	      y2 = y1 + PANGO_PIXELS (metrics->ascent + metrics->descent);
+	    }
+	  else
+	    {
+	      y2 = glyph_y;
+	      y1 = y2 - PANGO_UNKNOWN_GLYPH_HEIGHT;
+	    }
+
+	  x1 = glyph_x;
+	  x2 = x1 + PANGO_PIXELS (glyphs->glyphs[i].geometry.width);
+
           baseline = glyph_y;
 	  stroke_thick = MAX ((int) (0.5 + 0.075 * (y2 - y1)), 1);
           
-          wc = glyph & (~PANGO_X_UNKNOWN_FLAG);
+	  if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
+	    wc = glyph & (~PANGO_GLYPH_UNKNOWN_FLAG);
+	  else
+	    wc = 0;
 
           switch (wc)
             {
@@ -520,44 +572,6 @@ pango_x_render  (Display           *display,
 
 	  pango_font_metrics_unref (metrics);
         }
-      else
-	{
-	  guint16 index = PANGO_X_GLYPH_INDEX (glyph);
-	  guint16 subfont_index = PANGO_X_GLYPH_SUBFONT (glyph);
-	  PangoXSubfontInfo *subfont;
-      
-	  subfont = pango_x_find_subfont (font, subfont_index);
-	  if (subfont)
-	    {
-	      fs = pango_x_get_font_struct (font, subfont);
-	      if (!fs)
-		continue;
-	      
-	      if (fs->fid != old_fid)
-		{
-		  FLUSH;
-		  XSetFont (display, gc, fs->fid);
-		  old_fid = fs->fid;
-		}
-
-	      if (charcount == G_N_ELEMENTS (xcharbuffer) ||
-		  (charcount > 0 && (glyph_y != glyph_y0 ||
-				     glyph_x != expected_x)))
-		FLUSH;
-
-	      if (charcount == 0)
-		{
-		  glyph_x0 = glyph_x;
-		  glyph_y0 = glyph_y;
-		}
-	      xcharbuffer[charcount].byte1 = index / 256;
-	      xcharbuffer[charcount].byte2 = index % 256;
-
-	      expected_x = glyph_x + XTextWidth16 (fs, &xcharbuffer[charcount], 1);
-
-	      charcount++;
-	    }
-	}
 
     next_glyph:
       x_off += glyphs->glyphs[i].geometry.width;
@@ -576,56 +590,15 @@ pango_x_font_get_glyph_extents  (PangoFont      *font,
   XCharStruct *cs;
   PangoXSubfontInfo *subfont;
 
-  if (glyph & PANGO_X_UNKNOWN_FLAG)
+  if (glyph == PANGO_GLYPH_EMPTY)
     {
-      PangoFontMetrics *metrics = pango_font_get_metrics (font,
-							  pango_language_from_string ("en"));
-      gunichar wc;
-      gdouble width_factor;
-      int w;
-      
-      wc = glyph & (~PANGO_X_UNKNOWN_FLAG);
-          
-      switch (wc)
-        {
-        case '\n':
-        case '\r':
-        case 0x2028: /* Line separator */
-        case 0x2029: /* Paragraph separator */
-          {
-#define MAGIC_FACTOR 1.2
-
-            /* carriage-return thingy */
-	    width_factor = MAGIC_FACTOR;
-	    break;
-	  }
-	default:
-	  {
-	    /* Unknown glyph square */
-	    width_factor = 0.7;
-	  }
-	}
-	    
-      w = metrics->approximate_char_width * width_factor;
-      w = PANGO_SCALE * PANGO_PIXELS (w);
-            
       if (ink_rect)
-	{
-	  ink_rect->x = 0;
-	  ink_rect->width = w;
-	  ink_rect->y = - metrics->ascent;
-	  ink_rect->height = metrics->ascent + metrics->descent;
-	}
+	ink_rect->x = ink_rect->width = ink_rect->y = ink_rect->height = 0;
       if (logical_rect)
-	{
-	  logical_rect->x = 0;
-	  logical_rect->width = w;
-	  logical_rect->y = - metrics->ascent;
-	  logical_rect->height = metrics->ascent + metrics->descent;
-	}
-      
+	logical_rect->x = logical_rect->width = logical_rect->y = logical_rect->height = 0;
+      return;
     }
-  else if (glyph && pango_x_find_glyph (font, glyph, &subfont, &cs))
+  if ((glyph & PANGO_GLYPH_UNKNOWN_FLAG) == 0 && pango_x_find_glyph (font, glyph, &subfont, &cs))
     {
       if (ink_rect)
 	{
@@ -644,19 +617,66 @@ pango_x_font_get_glyph_extents  (PangoFont      *font,
     }
   else
     {
-      if (ink_rect)
-	{
-	  ink_rect->x = 0;
-	  ink_rect->width = 0;
-	  ink_rect->y = 0;
-	  ink_rect->height = 0;
+      PangoFontMetrics *metrics;
+      gunichar wc;
+      gdouble width_factor;
+      int w;
+      
+      if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
+	wc = glyph & (~PANGO_GLYPH_UNKNOWN_FLAG);
+      else
+        wc = 0;
+          
+      switch (wc)
+        {
+        case '\n':
+        case '\r':
+        case 0x2028: /* Line separator */
+        case 0x2029: /* Paragraph separator */
+          {
+#define MAGIC_FACTOR 1.2
+
+            /* carriage-return thingy */
+	    width_factor = MAGIC_FACTOR;
+	    break;
+	  }
+	default:
+	  {
+	    /* Unknown glyph square */
+	    width_factor = 1.0;
+	  }
 	}
-      if (logical_rect)
+	    
+      metrics = pango_font_get_metrics (font, NULL);
+
+      if (metrics)
+        {
+	  w = metrics->approximate_char_width * width_factor;
+	  w = PANGO_SCALE * PANGO_PIXELS (w);
+		
+	  if (ink_rect)
+	    {
+	      ink_rect->x = PANGO_SCALE;
+	      ink_rect->width = w - 2 * PANGO_SCALE;
+	      ink_rect->y = - (metrics->ascent - PANGO_SCALE);
+	      ink_rect->height = metrics->ascent + metrics->descent - 2 * PANGO_SCALE;
+	    }
+	  if (logical_rect)
+	    {
+	      logical_rect->x = 0;
+	      logical_rect->width = w;
+	      logical_rect->y = - metrics->ascent;
+	      logical_rect->height = metrics->ascent + metrics->descent;
+	    }
+
+	  pango_font_metrics_unref (metrics);
+	}
+      else
 	{
-	  logical_rect->x = 0;
-	  logical_rect->width = 0;
-	  logical_rect->y = 0;
-	  logical_rect->height = 0;
+	  if (ink_rect)
+	    ink_rect->x = ink_rect->y = ink_rect->height = ink_rect->width = 0;
+	  if (logical_rect)
+	    logical_rect->x = logical_rect->y = logical_rect->height = logical_rect->width = 0;
 	}
     }
 }
@@ -703,7 +723,7 @@ itemize_string_foreach (PangoFont     *font,
   PangoGlyphString *glyph_str = pango_glyph_string_new ();
   PangoEngineShape *shaper, *last_shaper;
   int last_level;
-  long n_chars, i;
+  int i;
   guint8 *embedding_levels;
   PangoDirection base_dir = PANGO_DIRECTION_LTR;
   gboolean finished = FALSE;
@@ -841,7 +861,12 @@ get_font_metrics_from_subfonts (PangoFont        *font,
   if (n_avg_widths)
     metrics->approximate_char_width = total_avg_widths / n_avg_widths;
   else
-    metrics->approximate_char_width = 10 * PANGO_SCALE;
+    metrics->approximate_char_width = PANGO_UNKNOWN_GLYPH_WIDTH * PANGO_SCALE;
+  if (metrics->ascent + metrics->descent == 0)
+    {
+      metrics->ascent = PANGO_UNKNOWN_GLYPH_HEIGHT * PANGO_SCALE;
+      metrics->descent = 0;
+    }
 }
 
 static void
@@ -852,12 +877,19 @@ get_subfonts_foreach (PangoFont      *font,
   GSList **subfonts = data;
   PangoGlyph glyph = glyph_info->glyph;
 
-  if ((glyph & PANGO_X_UNKNOWN_FLAG) == 0)
-    {
-      PangoXSubfont subfont = PANGO_X_GLYPH_SUBFONT (glyph);
-      if (!g_slist_find (*subfonts, GUINT_TO_POINTER ((guint)subfont)))
-	*subfonts = g_slist_prepend (*subfonts, GUINT_TO_POINTER ((guint)subfont));
-    }
+  if (glyph == PANGO_GLYPH_EMPTY)
+    return;
+
+  /* Use an arbitrary subfont for unknown glyphs...*/
+  if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
+    if (((PangoXFont *)font)->n_subfonts > 0)
+      glyph = PANGO_X_MAKE_GLYPH (1, 0);
+    else
+      return;
+
+  PangoXSubfont subfont = PANGO_X_GLYPH_SUBFONT (glyph);
+  if (!g_slist_find (*subfonts, GUINT_TO_POINTER ((guint)subfont)))
+    *subfonts = g_slist_prepend (*subfonts, GUINT_TO_POINTER ((guint)subfont));
 }
 
 /* Get composite font metrics for all subfonts resulting from shaping
@@ -924,15 +956,20 @@ pango_x_font_get_metrics (PangoFont        *font,
 
   if (!tmp_list)
     {
-      info = g_new (PangoXMetricsInfo, 1);
-      info->sample_str = sample_str;
-      info->metrics = pango_font_metrics_new ();
+      PangoFontMetrics *metrics;
+
+      info = g_new0 (PangoXMetricsInfo, 1);
 
       xfont->metrics_by_lang = g_slist_prepend (xfont->metrics_by_lang, info);
 
-      get_font_metrics_from_string (font, language, sample_str, info->metrics);
+      info->sample_str = sample_str;
+      metrics = pango_font_metrics_new ();
 
-      info->metrics->approximate_digit_width = get_total_width_for_string (font, language, "0123456789") / 10;
+      get_font_metrics_from_string (font, language, sample_str, metrics);
+
+      metrics->approximate_digit_width = get_total_width_for_string (font, language, "0123456789") / 10;
+
+      info->metrics = metrics;
     }
       
   return pango_font_metrics_ref (info->metrics);
@@ -1402,7 +1439,7 @@ pango_x_find_glyph (PangoFont *font,
  * @font: a #PangoFont.
  * 
  * Returns the index of a glyph suitable for drawing unknown characters;
- * you should generally use pango_x_font_get_unknown_glyph_instead,
+ * you should generally use PANGO_GET_UNKNOWN_GLYPH() instead,
  * since that may return a glyph that provides a better representation
  * of a particular char. (E.g., by showing hex digits, or a glyph
  * representive of a certain Unicode range.)
@@ -1412,7 +1449,7 @@ pango_x_find_glyph (PangoFont *font,
 PangoGlyph
 pango_x_get_unknown_glyph (PangoFont *font)
 {
-  return PANGO_X_UNKNOWN_FLAG;
+  return PANGO_GET_UNKNOWN_GLYPH (0);
 }
 
 /**
@@ -1546,7 +1583,7 @@ pango_x_render_layout_line (Display          *display,
  * @x:         the x position of the left of the layout (in pixels).
  * @y:         the y position of the top of the layout (in pixels).
  *
- * Renders a #PangoLayoutLine onto an X drawable.
+ * Renders a #PangoLayout onto an X drawable.
  */
 void 
 pango_x_render_layout (Display         *display,
@@ -1709,7 +1746,7 @@ pango_x_find_first_subfont (PangoFont      *font,
  * 
  * This is a simple fallback shaper, that can be used
  * if no subfont that supports a given script is found. 
- * For every character in @text, it puts the Unknown glyph.
+ * For every character in @text, it puts the unknown glyph.
  */
 void 
 pango_x_fallback_shape (PangoFont        *font, 
@@ -1733,10 +1770,12 @@ pango_x_fallback_shape (PangoFont        *font,
   for (i = 0; i < n_chars; i++)
     {
       glyphs->glyphs[i].glyph = unknown_glyph;
+
       glyphs->glyphs[i].geometry.x_offset = 0;
       glyphs->glyphs[i].geometry.y_offset = 0;
       glyphs->glyphs[i].geometry.width = logical_rect.width;
-      glyphs->log_clusters[i] = 0;
+
+      glyphs->log_clusters[i] = p - text;
       
       p = g_utf8_next_char (p);
     }
@@ -1750,13 +1789,13 @@ pango_x_fallback_shape (PangoFont        *font,
  * Returns the index of a glyph suitable for drawing @wc as an
  * unknown character.
  *
+ * Use PANGO_GET_UNKNOWN_GLYPH() instead.
+ *
  * Return value: a glyph index into @font.
  */
 PangoGlyph
 pango_x_font_get_unknown_glyph (PangoFont *font,
                                 gunichar   wc)
 {
-  g_return_val_if_fail (PANGO_IS_FONT (font), 0);
-
-  return PANGO_X_UNKNOWN_FLAG | wc;
+  return PANGO_GET_UNKNOWN_GLYPH (wc);
 }
